/// <summary>
/// ****************************************************************************
/// @FILE:      program.cs
///
/// @BRIEF:     The main entry point for the ContLog application.
///
/// @HISTORY:   2016-03-25  J.Bendor    Created
/// 
/// @COPYRIGHT: ACCES-IO, All Rights Reserved.
/// ****************************************************************************
/// </summary>

using System;
using System.IO;
using System.Threading;
using System.Windows.Forms;


namespace AppContLog
{
    static class Program
    {
        /// <summary>	
        /// ***************************************************************************************
        /// The main entry point for the application.
        /// ***************************************************************************************
        /// </summary>
        [STAThread]
        public static void Main(string[] args)
        {
            Application.EnableVisualStyles();

            Application.SetCompatibleTextRenderingDefault(false);

            try
            {
                Application.SetUnhandledExceptionMode(UnhandledExceptionMode.ThrowException);

                Application.ThreadException += new ThreadExceptionEventHandler(handle_GUI_Thread_Exceptions);

                AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(handle_Non_GUI_Thread_Exceptions);

                MainForm.TMainForm main_form = new MainForm.TMainForm();

                Application.Run(main_form);
            }
            catch (Exception ex)
            {
                Report_Unhandled_Exception(ex);
            }
        }

        private static void handle_GUI_Thread_Exceptions(object sender, ThreadExceptionEventArgs ev_args)
        {
            Report_Unhandled_Exception(ev_args.Exception);
        }

        private static void handle_Non_GUI_Thread_Exceptions(object sender, UnhandledExceptionEventArgs ev_args)
        {
            Report_Unhandled_Exception( (Exception) ev_args.ExceptionObject );
        }

        public static void Report_Unhandled_Exception(Exception ex)
        {
            string str_now = DateTime.Now.ToString("yyyyMMdd_HHmmss");

            string str_note = ex.Message + "\r\n" + ex.StackTrace;

            string str_path = Path.Combine(Path.GetTempPath(), "exception_" + str_now + ".TXT");

            System.IO.StreamWriter file_writer = new StreamWriter(str_path);

            file_writer.Write(str_note);

            if (ex.InnerException != null)
            {
                file_writer.Write( "\r\n !INNER \r\n" + ex.InnerException.StackTrace );
            }

            file_writer.Close();

            MessageBox.Show("LOGGED: " + str_path + "\r\n\n" + str_note, "FATAL EXCEPTION");
        }

    } // ends class Program

} // ends namespace AppContLog
