/// <summary>
/// ****************************************************************************
/// @FILE:      DetectForm.cs
///
/// @BRIEF:     implements the DetectForm dialog for the ContLog app.
///
/// @HISTORY:   2016-03-25  J.Bendor    Created from "DetectUnit.pas"
/// 
/// @COPYRIGHT: ACCES-IO, All Rights Reserved.
/// ****************************************************************************
/// </summary>

using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AIOUSB_API;


namespace DetectForm
{
    public partial class TDetectForm : Form
    {
        private uint mDeviceIndexNow = 0;
        private bool[] mDeviceExists = new bool[AIOUSB.MAX_NUM_DEVICES + 1];

        public TDetectForm()
        {
            InitializeComponent();
        }

        private void TDetectForm_FormLoad(object sender, EventArgs evarg)
        {
            try
            {
                UpdateDeviceList();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        private void btnSelect_Click(object sender, EventArgs evarg)
        {
            int idx = DeviceList.SelectedIndex;

            if ((idx < 0) || (! mDeviceExists[idx]))
            {
                Console.Beep();
                return;
            }

            mDeviceIndexNow = (uint) idx;
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void btnDetect_Click(object sender, EventArgs evarg)
        {
            UpdateDeviceList();
        }

        public uint DeviceIndex
        {
            get { return mDeviceIndexNow; }
        }

        public uint UpdateDeviceList()
        {
            uint uDevicesMask = AIOUSB.GetDevices();
            uint uNumDevFound = 0;

            DeviceList.Items.Clear();

            for (uint idx = 0, bit = 1; idx <= AIOUSB.MAX_NUM_DEVICES;  ++idx, bit += bit)
            {
                mDeviceExists[idx] = false;

                if ((uDevicesMask & bit) == 0)
                {
                    DeviceList.Items.Add(string.Format("#{0}  (no-device)", idx));
                    continue;
                }

                uint   uPID = 0;
                string Name = "?";

                uint status = AIOUSB.QueryDeviceInfo(idx, out uPID, out Name);

                switch (uPID)   // we only want USB-AIx Family
                {
				case 0x8140: case 0x8141: case 0x8142: case 0x8143: case 0x8144: case 0x815F:
				case 0x8145: case 0x8146: case 0x8147: case 0x8148: case 0x8149:
				case 0x814A: case 0x814B: case 0x814C: case 0x814D: case 0x814E:
				case 0x814F: case 0x8150: case 0x8151: case 0x8152: case 0x8153:
				case 0x8154: case 0x8155: case 0x8156: case 0x8157: case 0x8158:
				case 0x8159: case 0x815A: case 0x815B: case 0x815C: case 0x815D:
                        mDeviceExists[idx] = true;
                        uNumDevFound += 1;
                        Name += string.Format("  uPID={0:X4}", uPID);
				break;

				default:
					Name += string.Format("  uPID={0:X4}  ignored", uPID);
				break;
                }

                DeviceList.Items.Add( string.Format("#{0}  {1}", idx, Name) );
            }

            if (uNumDevFound > 0)
            {
                DeviceList.SelectedIndex = 0;
                StatusMemo.Text = "Select the device you want to work with and click Go.";
            }
            else
            {
                StatusMemo.Text = "No compatible devices found; your device may not be installed, or may not be installed properly. Click Redetect to check again.";
            }

            return uNumDevFound;
        }

    } // ends class TDetectForm

} // ends namespace DetectForm
